#include "init.h"

extern void setSeed(long);
extern void clearStringCache(void);
extern void initStringCache(void);
extern void initColors(void);
extern TTF_Font *loadFont(char *, int);
extern void closeFont(TTF_Font *);
extern void freeAnimations(void);
extern void loadSound(char *, int);
extern void loadMusic(char *);
extern void playMusic(void);
extern void loadSettings(void);
extern void saveSettings(void);
extern void freeMenus(void);
extern void saveHighScores(void);

void setupUserHomeDirectory(void);

void init()
{
	int joysticks, buttons;

	if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_JOYSTICK|SDL_INIT_AUDIO) < 0)
	{
		printf("Couldn't initialize SDL: %s\n", SDL_GetError());

		exit(1);
	}

	if (TTF_Init() < 0)
	{
		printf("Couldn't initialize SDL TTF: %s\n", SDL_GetError());

		exit(1);
	}

	game.online = 1;

	if (SDLNet_Init() < 0)
	{
		printf("Could not initialize SDL_Net: %s\n", SDLNet_GetError());

		game.online = 0;
	}

	screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, 0, SDL_HWPALETTE);

	if (screen == NULL)
	{
		printf("Couldn't set screen mode to %d x %d: %s\n", SCREEN_WIDTH, SCREEN_HEIGHT, SDL_GetError());

		exit(1);
	}

	joysticks = SDL_NumJoysticks();

	if (joysticks > 0)
	{
		printf("Found %d joysticks Opening Joystick #1: %s\n", joysticks, SDL_JoystickName(0));

		joystick = SDL_JoystickOpen(0);

		buttons = SDL_JoystickNumButtons(joystick);

		printf("Joystick has %d buttons\n", buttons);

		printf("Joystick has %d axes\n", SDL_JoystickNumAxes(joystick));

		buttonState = (int *)malloc(buttons * sizeof(int));

		if (buttonState == NULL)
		{
			printf("Failed to allocate %d bytes for buttons!\n", buttons);

			exit(1);
		}
	}

	game.audio = 1;

	if (Mix_OpenAudio(22050, AUDIO_S16SYS, 2, 4096) != 0)
	{
		printf("Could not open audio: %s\n", Mix_GetError());
		printf("Sound will be disabled\n");

		game.audio = 0;
	}

	SDL_ShowCursor(SDL_DISABLE);

	SDL_WM_SetCaption("Random Shooter", NULL);

	setSeed(time(NULL));

	standardFont = loadFont(INSTALL_DIR"font/blackWolf.ttf", 16);
	largeFont = loadFont(INSTALL_DIR"font/blackWolf.ttf", 48);

	initColors();

	initStringCache();

	#ifdef UNIX
		setupUserHomeDirectory();
	#else
		strcpy(game.savePath, "");
	#endif

	loadSettings();

	loadMusic(INSTALL_DIR"music/17.mod");

	playMusic();

	game.ticks = SDL_GetTicks() + 16;
}

void limitFPS()
{
	unsigned long ticks = SDL_GetTicks();

	if (ticks < game.ticks)
	{
		SDL_Delay(game.ticks - ticks);
	}

	game.ticks = SDL_GetTicks() + 16;
}

void cleanup()
{
	int i;

	if (joystick != NULL)
	{
		printf("Closing joystick %s\n", SDL_JoystickName(0));

		SDL_JoystickClose(joystick);
	}

	if (buttonState != NULL)
	{
		printf("Freeing buttons\n");

		free(buttonState);
	}

	if (game.audio == 1)
	{
		for (i=0;i<MAX_SOUNDS;i++)
		{
			if (sounds.effect[i] != NULL)
			{
				printf("Freeing Sound %d\n", i);

				Mix_FreeChunk(sounds.effect[i]);
			}
		}

		if (music != NULL)
		{
			printf("Halting Music\n");

			Mix_HaltMusic();

			printf("Freeing Music\n");

			Mix_FreeMusic(music);
		}

		printf("Closing Audio\n");

		Mix_CloseAudio();
	}

	printf("Freeing Menus\n");

	freeMenus();

	printf("Freeing Animations\n");

	freeAnimations();

	printf("Freeing Fonts\n");

	closeFont(standardFont);
	closeFont(largeFont);

	clearStringCache();

	printf("Closing TTF\n");

	TTF_Quit();

	printf("Saving Highscores\n");

	saveHighScores();

	printf("Saving Settings\n");

	saveSettings();

	printf("Exiting\n");

	SDL_Quit();
}

#ifdef UNIX
void setupUserHomeDirectory()
{
	char *userHome;
	char *name;
	struct passwd *pass;
	char dir[MAX_PATH];
	char gameSavePath[MAX_PATH];

	name = getlogin();

	strcpy(dir, "");

	if (name != NULL)
	{
		pass = getpwnam(name);
	}

	else
	{
		pass = getpwuid(geteuid());
	}

	if (pass == NULL)
	{
		printf("Couldn't determine the user home directory. Exiting.\n");
		exit(1);
	}

	userHome = pass->pw_dir;

	printf("User Home = %s\n", userHome);

	sprintf(dir, "%s/.parallelrealities", userHome);

	if ((mkdir(dir, S_IRWXU|S_IRWXG|S_IROTH|S_IXOTH) != 0) && (errno != EEXIST))
	{
		printf("Couldn't create required directory '%s'", dir);

		exit(1);
	}

	sprintf(dir, "%s/.parallelrealities/randomshooter", userHome);

	if ((mkdir(dir, S_IRWXU|S_IRWXG|S_IROTH|S_IXOTH) != 0) && (errno != EEXIST))
	{
		printf("Couldn't create required directory '%s'", dir);

		exit(1);
	}

	sprintf(gameSavePath, "%s/.parallelrealities/randomshooter/", userHome);

	strcpy(game.savePath, gameSavePath);
}


#endif
